#!/usr/bin/python

# SPDX-FileCopyrightText: 2025-2026 Arcangelo Massari <arcangelo.massari@unibo.it>
#
# SPDX-License-Identifier: ISC

# -*- coding: utf-8 -*-
import os
import unittest

from rdflib import Graph, Literal, Namespace, URIRef, XSD

from oc_ocdm.graph.graph_set import GraphSet
from oc_ocdm.prov.prov_set import ProvSet
from oc_ocdm.support.support import (build_graph_from_results, create_date,
                                     create_literal, find_paths,
                                     get_datatype_from_iso_8601,
                                     get_ordered_contributors_from_br,
                                     normalize_graph_literals,
                                     sparql_binding_to_term)


class TestSupport(unittest.TestCase):
    resp_agent = 'http://resp_agent.test/'
    pro = Namespace("http://purl.org/spar/pro/")

    def setUp(self):
        self.graph_set = GraphSet("http://test/", "./info_dir/", "", False)
        self.prov_set = ProvSet(self.graph_set, "http://test/", "./info_dir/", False)

        self.br = self.graph_set.add_br(self.resp_agent)

    def _prepare_ordered_authors_list(self, list_len):
        # First of all, we must cleanup the GraphSet:
        self.br.remove_contributor()

        for ar in self.graph_set.get_ar():
            del self.graph_set.res_to_entity[ar.res]
        for ra in self.graph_set.get_ra():
            del self.graph_set.res_to_entity[ra.res]

        # Then, we initialize a new well-formed linked list of authors:
        ar_ordered_list = []

        for i in range(list_len):
            ra = self.graph_set.add_ra(self.resp_agent)

            ar = self.graph_set.add_ar(self.resp_agent)
            ar.create_author()
            ar.is_held_by(ra)

            self.br.has_contributor(ar)
            ar_ordered_list.append(ar)

        # Here each node of the list gets linked to the next one:
        for i in range(list_len - 1):
            ar_ordered_list[i].has_next(ar_ordered_list[i + 1])

        return ar_ordered_list

    @staticmethod
    def _extract_ra_list(ar_list):
        # Here the RA list is built and returned:
        ra_list = []
        for i in range(len(ar_list)):
            ra = ar_list[i].get_is_held_by()
            if ra is not None:
                ra_list.append(ra)

        return ra_list

    def test_get_ordered_contributors_from_br(self):
        list_len = 100
        with self.subTest("Empty linked list"):
            result = get_ordered_contributors_from_br(self.br, self.pro.author)

            self.assertIsNotNone(result)
            self.assertListEqual([], result)

        with self.subTest("Well-formed linked list"):
            correct_list = self._prepare_ordered_authors_list(list_len)
            result = get_ordered_contributors_from_br(self.br, self.pro.author)

            self.assertIsNotNone(result)
            ar_list = self._extract_ra_list(correct_list)
            self.assertListEqual(ar_list, result)

        with self.subTest("Linked list with a loop"):
            correct_list = self._prepare_ordered_authors_list(list_len)
            # Here we corrupt the well-formed linked list introducing a loop:
            correct_list[80].has_next(correct_list[50])

            self.assertRaises(ValueError, get_ordered_contributors_from_br, self.br, self.pro.author)

        with self.subTest("Linked list split in two sub-lists"):
            correct_list = self._prepare_ordered_authors_list(list_len)
            # Here we corrupt the well-formed linked list introducing a loop:
            correct_list[64].remove_next()

            self.assertRaises(ValueError, get_ordered_contributors_from_br, self.br, self.pro.author)
    
    def test_find_paths(self):
        cur_dir_path, cur_file_path = find_paths(
            res = URIRef('https://w3id.org/oc/meta/br/060169'),
            base_dir = os.path.join('support', 'test', 'data', 'rdf'),
            base_iri = 'https://w3id.org/oc/meta',
            default_dir = '_',
            dir_split = 10000,
            n_file_item = 1000,
            is_json = True)
        self.assertEqual((cur_dir_path, cur_file_path), (os.path.join('support', 'test', 'data', 'rdfbr', '060', '10000'), os.path.join('support', 'test', 'data', 'rdfbr', '060', '10000', '1000.json')))

    def test_find_paths_prov(self):
        cur_dir_path, cur_file_path = find_paths(
            res = URIRef('https://w3id.org/oc/meta/br/060165/prov/se/1'),
            base_dir = os.path.join('support', 'test', 'data', 'rdf'),
            base_iri = 'https://w3id.org/oc/meta',
            default_dir = '_',
            dir_split = 10000,
            n_file_item = 1000,
            is_json = True)
        self.assertEqual((cur_dir_path, cur_file_path), (os.path.join('support', 'test', 'data', 'rdfbr', '060', '10000', '1000', 'prov'), os.path.join('support', 'test', 'data', 'rdfbr', '060', '10000', '1000', 'prov', 'se.json')))


class TestSparqlBindingToTerm(unittest.TestCase):

    def test_uri_binding(self):
        binding = {'type': 'uri', 'value': 'http://example.org/resource'}
        result = sparql_binding_to_term(binding)
        self.assertEqual(result, URIRef('http://example.org/resource'))

    def test_literal_with_datatype(self):
        binding = {
            'type': 'literal',
            'value': '42',
            'datatype': 'http://www.w3.org/2001/XMLSchema#integer'
        }
        result = sparql_binding_to_term(binding)
        self.assertEqual(result, Literal('42', datatype=XSD.integer))

    def test_literal_with_language(self):
        binding = {'type': 'literal', 'value': 'hello', 'xml:lang': 'en'}
        result = sparql_binding_to_term(binding)
        self.assertEqual(result, Literal('hello', lang='en'))
        self.assertIsInstance(result, Literal)
        assert isinstance(result, Literal)
        self.assertEqual(result.language, 'en')

    def test_simple_literal_normalized_to_xsd_string(self):
        binding = {'type': 'literal', 'value': 'plain text'}
        result = sparql_binding_to_term(binding)
        self.assertEqual(result, Literal('plain text', datatype=XSD.string))
        self.assertIsInstance(result, Literal)
        assert isinstance(result, Literal)
        self.assertEqual(result.datatype, XSD.string)


class TestNormalizeGraphLiterals(unittest.TestCase):

    def test_simple_literal_gets_xsd_string(self):
        g = Graph()
        s = URIRef('http://example.org/s')
        p = URIRef('http://example.org/p')
        g.add((s, p, Literal('plain')))
        normalize_graph_literals(g)
        obj = list(g.objects(s, p))[0]
        assert isinstance(obj, Literal)
        self.assertEqual(obj.datatype, XSD.string)

    def test_typed_literal_unchanged(self):
        g = Graph()
        s = URIRef('http://example.org/s')
        p = URIRef('http://example.org/p')
        original = Literal('42', datatype=XSD.integer)
        g.add((s, p, original))
        normalize_graph_literals(g)
        obj = list(g.objects(s, p))[0]
        assert isinstance(obj, Literal)
        self.assertEqual(obj.datatype, XSD.integer)

    def test_lang_literal_unchanged(self):
        g = Graph()
        s = URIRef('http://example.org/s')
        p = URIRef('http://example.org/p')
        original = Literal('ciao', lang='it')
        g.add((s, p, original))
        normalize_graph_literals(g)
        obj = list(g.objects(s, p))[0]
        assert isinstance(obj, Literal)
        self.assertEqual(obj.language, 'it')
        self.assertIsNone(obj.datatype)

    def test_empty_graph(self):
        g = Graph()
        normalize_graph_literals(g)
        self.assertEqual(len(g), 0)


class TestCreateDate(unittest.TestCase):

    def test_none_returns_none(self):
        self.assertIsNone(create_date(None))

    def test_empty_list_returns_none(self):
        self.assertIsNone(create_date([]))

    def test_year_only(self):
        self.assertEqual(create_date([2023]), '2023')

    def test_year_month(self):
        self.assertEqual(create_date([2023, 6]), '2023-06')

    def test_full_date(self):
        self.assertEqual(create_date([2023, 6, 15]), '2023-06-15')

    def test_full_date_jan_1_returns_year_month(self):
        self.assertEqual(create_date([2023, 1, 1]), '2023-01')

    def test_three_elements_month_none_day_none_returns_year(self):
        self.assertEqual(create_date([2023, None, None]), '2023')

    def test_three_elements_with_month_and_day_1(self):
        self.assertEqual(create_date([2023, 3, 1]), '2023-03-01')

    def test_first_element_none(self):
        self.assertIsNone(create_date([None]))


class TestGetDatatypeFromIso8601(unittest.TestCase):

    def test_full_date(self):
        dt, val = get_datatype_from_iso_8601('2023-06-15')
        self.assertEqual(dt, XSD.date)
        self.assertEqual(val, '2023-06-15')

    def test_year_month(self):
        dt, val = get_datatype_from_iso_8601('2023-06')
        self.assertEqual(dt, XSD.gYearMonth)
        self.assertEqual(val, '2023-06')

    def test_year_only(self):
        dt, val = get_datatype_from_iso_8601('2023')
        self.assertEqual(dt, XSD.gYear)
        self.assertEqual(val, '2023')

    def test_truncates_after_10_chars(self):
        dt, val = get_datatype_from_iso_8601('2023-06-15T10:30:00')
        self.assertEqual(dt, XSD.date)
        self.assertEqual(val, '2023-06-15')

    def test_invalid_string_raises(self):
        with self.assertRaises(ValueError):
            get_datatype_from_iso_8601('not-a-date')


class TestCreateLiteral(unittest.TestCase):

    def test_adds_literal_with_default_xsd_string(self):
        g = Graph()
        res = URIRef('http://example.org/s')
        p = URIRef('http://example.org/p')
        create_literal(g, res, p, 'hello')
        obj = list(g.objects(res, p))[0]
        assert isinstance(obj, Literal)
        self.assertEqual(str(obj), 'hello')
        self.assertEqual(obj.datatype, XSD.string)

    def test_adds_literal_with_explicit_datatype(self):
        g = Graph()
        res = URIRef('http://example.org/s')
        p = URIRef('http://example.org/p')
        create_literal(g, res, p, '42', dt=XSD.integer)
        obj = list(g.objects(res, p))[0]
        assert isinstance(obj, Literal)
        self.assertEqual(obj.datatype, XSD.integer)

    def test_empty_string_adds_nothing(self):
        g = Graph()
        res = URIRef('http://example.org/s')
        p = URIRef('http://example.org/p')
        create_literal(g, res, p, '')
        self.assertEqual(len(g), 0)

    def test_whitespace_only_adds_nothing(self):
        g = Graph()
        res = URIRef('http://example.org/s')
        p = URIRef('http://example.org/p')
        create_literal(g, res, p, '   ')
        self.assertEqual(len(g), 0)


class TestBuildGraphFromResults(unittest.TestCase):

    def test_uri_triple(self):
        results = [{
            's': {'type': 'uri', 'value': 'http://example.org/s'},
            'p': {'type': 'uri', 'value': 'http://example.org/p'},
            'o': {'type': 'uri', 'value': 'http://example.org/o'}
        }]
        g = build_graph_from_results(results)
        self.assertEqual(len(g), 1)
        s, p, o = next(iter(g))
        self.assertEqual(s, URIRef('http://example.org/s'))
        self.assertEqual(o, URIRef('http://example.org/o'))

    def test_literal_object_with_datatype(self):
        results = [{
            's': {'type': 'uri', 'value': 'http://example.org/s'},
            'p': {'type': 'uri', 'value': 'http://example.org/p'},
            'o': {'type': 'literal', 'value': '42', 'datatype': 'http://www.w3.org/2001/XMLSchema#integer'}
        }]
        g = build_graph_from_results(results)
        o = list(g.objects())[0]
        assert isinstance(o, Literal)
        self.assertEqual(o.datatype, XSD.integer)

    def test_literal_object_without_datatype_gets_xsd_string(self):
        results = [{
            's': {'type': 'uri', 'value': 'http://example.org/s'},
            'p': {'type': 'uri', 'value': 'http://example.org/p'},
            'o': {'type': 'literal', 'value': 'plain'}
        }]
        g = build_graph_from_results(results)
        o = list(g.objects())[0]
        assert isinstance(o, Literal)
        self.assertEqual(o.datatype, XSD.string)

    def test_literal_object_with_language(self):
        results = [{
            's': {'type': 'uri', 'value': 'http://example.org/s'},
            'p': {'type': 'uri', 'value': 'http://example.org/p'},
            'o': {'type': 'literal', 'value': 'bonjour', 'xml:lang': 'fr'}
        }]
        g = build_graph_from_results(results)
        o = list(g.objects())[0]
        assert isinstance(o, Literal)
        self.assertEqual(o.language, 'fr')

    def test_empty_results(self):
        g = build_graph_from_results([])
        self.assertEqual(len(g), 0)


if __name__ == '__main__':
    unittest.main()


#!/usr/bin/python

# SPDX-FileCopyrightText: 2025 Arcangelo Massari <arcangelo.massari@unibo.it>
#
# SPDX-License-Identifier: ISC

# -*- coding: utf-8 -*-
import unittest

from oc_ocdm.metadata.metadata_entity import MetadataEntity
from oc_ocdm.metadata.metadata_set import MetadataSet
from rdflib import XSD, Literal, URIRef


class TestDataset(unittest.TestCase):
    resp_agent = 'http://resp_agent.test/'

    @classmethod
    def setUpClass(cls) -> None:
        cls.metadata_set = MetadataSet("http://test/", "./info_dir/", False)

    def setUp(self):
        self.dataset = self.metadata_set.add_dataset("ocdmTest", self.resp_agent)
        self.sub_dataset = self.metadata_set.add_dataset("subDataset", self.resp_agent)
        self.di = self.metadata_set.add_di("ocdmTest", self.resp_agent)

    def test_has_title(self):
        title = "Resource"
        result = self.dataset.has_title(title)
        self.assertIsNone(result)

        triple = self.dataset.res, MetadataEntity.iri_title, Literal(title, datatype=XSD.string)
        self.assertIn(triple, self.dataset.g)

    def test_has_description(self):
        description = "Resource"
        result = self.dataset.has_description(description)
        self.assertIsNone(result)

        triple = self.dataset.res, MetadataEntity.iri_description, Literal(description, datatype=XSD.string)
        self.assertIn(triple, self.dataset.g)

    def test_has_publication_date(self):
        string = "2020-05-25T12:12:00"
        result = self.dataset.has_publication_date(string)
        self.assertIsNone(result)

        triple = self.dataset.res, MetadataEntity.iri_issued, Literal(string, datatype=XSD.dateTime,
                                                                      normalize=False)
        self.assertIn(triple, self.dataset.g)

    def test_has_modification_date(self):
        string = "2020-05-25T12:12:00"
        result = self.dataset.has_modification_date(string)
        self.assertIsNone(result)

        triple = self.dataset.res, MetadataEntity.iri_modified, Literal(string, datatype=XSD.dateTime,
                                                                        normalize=False)
        self.assertIn(triple, self.dataset.g)

    def test_has_keyword(self):
        keyword = "Resource"
        result = self.dataset.has_keyword(keyword)
        self.assertIsNone(result)

        triple = self.dataset.res, MetadataEntity.iri_keyword, Literal(keyword, datatype=XSD.string)
        self.assertIn(triple, self.dataset.g)

    def test_has_subject(self):
        subject = URIRef("http://subject/")
        result = self.dataset.has_subject(subject)
        self.assertIsNone(result)

        triple = self.dataset.res, MetadataEntity.iri_subject, subject
        self.assertIn(triple, self.dataset.g)

    def test_has_landing_page(self):
        page = URIRef("http://landing.page/")
        result = self.dataset.has_landing_page(page)
        self.assertIsNone(result)

        triple = self.dataset.res, MetadataEntity.iri_landing_page, page
        self.assertIn(triple, self.dataset.g)

    def test_has_sub_dataset(self):
        result = self.dataset.has_sub_dataset(self.sub_dataset)
        self.assertIsNone(result)

        triple = self.dataset.res, MetadataEntity.iri_subset, self.sub_dataset.res
        self.assertIn(triple, self.dataset.g)

    def test_has_sparql_endpoint(self):
        endpoint = URIRef("http://sparql/")
        result = self.dataset.has_sparql_endpoint(endpoint)
        self.assertIsNone(result)

        triple = self.dataset.res, MetadataEntity.iri_sparql_endpoint, endpoint
        self.assertIn(triple, self.dataset.g)

    def test_has_distribution(self):
        result = self.dataset.has_distribution(self.di)
        self.assertIsNone(result)

        triple = self.dataset.res, MetadataEntity.iri_distribution, self.di.res
        self.assertIn(triple, self.dataset.g)

    def test_merge_with_title(self):
        ds1 = self.metadata_set.add_dataset("ds1", self.resp_agent)
        ds2 = self.metadata_set.add_dataset("ds2", self.resp_agent)

        title = "Test Dataset"
        ds2.has_title(title)

        ds1.merge(ds2)

        self.assertEqual(ds1.get_title(), title)
        self.assertTrue(ds2.to_be_deleted)

    def test_merge_with_description(self):
        ds1 = self.metadata_set.add_dataset("ds1", self.resp_agent)
        ds2 = self.metadata_set.add_dataset("ds2", self.resp_agent)

        description = "Test description"
        ds2.has_description(description)

        ds1.merge(ds2)

        self.assertEqual(ds1.get_description(), description)

    def test_merge_with_dates(self):
        ds1 = self.metadata_set.add_dataset("ds1", self.resp_agent)
        ds2 = self.metadata_set.add_dataset("ds2", self.resp_agent)

        pub_date = "2020-01-01T00:00:00"
        mod_date = "2020-12-31T23:59:59"
        ds2.has_publication_date(pub_date)
        ds2.has_modification_date(mod_date)

        ds1.merge(ds2)

        self.assertEqual(ds1.get_publication_date(), pub_date)
        self.assertEqual(ds1.get_modification_date(), mod_date)

    def test_merge_with_keywords(self):
        ds1 = self.metadata_set.add_dataset("ds1", self.resp_agent)
        ds2 = self.metadata_set.add_dataset("ds2", self.resp_agent)

        ds2.has_keyword("keyword1")
        ds2.has_keyword("keyword2")

        ds1.merge(ds2)

        keywords = ds1.get_keywords()
        self.assertEqual(len(keywords), 2)

    def test_merge_with_subjects(self):
        ds1 = self.metadata_set.add_dataset("ds1", self.resp_agent)
        ds2 = self.metadata_set.add_dataset("ds2", self.resp_agent)

        subject1 = URIRef("http://subject1/")
        subject2 = URIRef("http://subject2/")
        ds2.has_subject(subject1)
        ds2.has_subject(subject2)

        ds1.merge(ds2)

        subjects = ds1.get_subjects()
        self.assertEqual(len(subjects), 2)

    def test_merge_basic(self):
        ds1 = self.metadata_set.add_dataset("ds1", self.resp_agent)
        ds2 = self.metadata_set.add_dataset("ds2", self.resp_agent)

        title = "Test"
        ds2.has_title(title)

        ds1.merge(ds2)
        self.assertTrue(ds2.to_be_deleted)

    def test_merge_with_sub_datasets(self):
        ds1 = self.metadata_set.add_dataset("ds1", self.resp_agent)
        ds2 = self.metadata_set.add_dataset("ds2", self.resp_agent)
        sub_ds = self.metadata_set.add_dataset("sub", self.resp_agent)

        ds2.has_sub_dataset(sub_ds)

        ds1.merge(ds2)

        sub_datasets = ds1.get_sub_datasets()
        self.assertEqual(len(sub_datasets), 1)

    def test_merge_with_sparql_endpoint(self):
        ds1 = self.metadata_set.add_dataset("ds1", self.resp_agent)
        ds2 = self.metadata_set.add_dataset("ds2", self.resp_agent)

        endpoint = URIRef("http://sparql.endpoint/")
        ds2.has_sparql_endpoint(endpoint)

        ds1.merge(ds2)

        self.assertEqual(ds1.get_sparql_endpoint(), endpoint)

    def test_merge_with_distributions(self):
        ds1 = self.metadata_set.add_dataset("ds1", self.resp_agent)
        ds2 = self.metadata_set.add_dataset("ds2", self.resp_agent)
        di = self.metadata_set.add_di("di", self.resp_agent)

        ds2.has_distribution(di)

        ds1.merge(ds2)

        distributions = ds1.get_distributions()
        self.assertEqual(len(distributions), 1)

    def test_get_title(self):
        title = "Test Dataset"
        self.dataset.has_title(title)
        self.assertEqual(self.dataset.get_title(), title)

    def test_get_description(self):
        description = "Test description"
        self.dataset.has_description(description)
        self.assertEqual(self.dataset.get_description(), description)

    def test_get_publication_date(self):
        pub_date = "2020-01-01T00:00:00"
        self.dataset.has_publication_date(pub_date)
        self.assertEqual(self.dataset.get_publication_date(), pub_date)

    def test_get_modification_date(self):
        mod_date = "2020-12-31T23:59:59"
        self.dataset.has_modification_date(mod_date)
        self.assertEqual(self.dataset.get_modification_date(), mod_date)

    def test_get_keywords(self):
        self.dataset.has_keyword("keyword1")
        self.dataset.has_keyword("keyword2")
        keywords = self.dataset.get_keywords()
        self.assertEqual(len(keywords), 2)

    def test_get_subjects(self):
        subject1 = URIRef("http://subject1/")
        subject2 = URIRef("http://subject2/")
        self.dataset.has_subject(subject1)
        self.dataset.has_subject(subject2)
        subjects = self.dataset.get_subjects()
        self.assertEqual(len(subjects), 2)


    def test_get_sub_datasets(self):
        self.dataset.has_sub_dataset(self.sub_dataset)
        sub_datasets = self.dataset.get_sub_datasets()
        self.assertEqual(len(sub_datasets), 1)

    def test_get_sparql_endpoint(self):
        endpoint = URIRef("http://sparql.endpoint/")
        self.dataset.has_sparql_endpoint(endpoint)
        self.assertEqual(self.dataset.get_sparql_endpoint(), endpoint)

    def test_get_distributions(self):
        self.dataset.has_distribution(self.di)
        distributions = self.dataset.get_distributions()
        self.assertEqual(len(distributions), 1)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/python

# SPDX-FileCopyrightText: 2020-2022 Simone Persiani <iosonopersia@gmail.com>
# SPDX-FileCopyrightText: 2023-2026 Arcangelo Massari <arcangelo.massari@unibo.it>
#
# SPDX-License-Identifier: ISC

# -*- coding: utf-8 -*-
from __future__ import annotations

from typing import TYPE_CHECKING, List, Set

from rdflib import URIRef

if TYPE_CHECKING:
    from typing import Tuple
    from oc_ocdm.abstract_entity import AbstractEntity

MAX_TRIPLES_PER_QUERY = 500


def _serialize_triples_to_nt(triples: Set) -> str:
    return "".join(f"{s.n3()} {p.n3()} {o.n3()} ." for s, p, o in triples)


def _chunk_set(data: Set, chunk_size: int) -> List[Set]:
    data_list = list(data)
    return [set(data_list[i:i + chunk_size]) for i in range(0, len(data_list), chunk_size)]


def get_delete_query(graph_iri: URIRef, data: Set) -> Tuple[List[str], int]:
    num_of_statements: int = len(data)
    if num_of_statements <= 0:
        return [], 0

    if num_of_statements <= MAX_TRIPLES_PER_QUERY:
        statements: str = _serialize_triples_to_nt(data)
        return [f"DELETE DATA {{ GRAPH <{graph_iri}> {{ {statements} }} }}"], num_of_statements

    chunks = _chunk_set(data, MAX_TRIPLES_PER_QUERY)
    queries = []
    for chunk in chunks:
        statements = _serialize_triples_to_nt(chunk)
        queries.append(f"DELETE DATA {{ GRAPH <{graph_iri}> {{ {statements} }} }}")
    return queries, num_of_statements


def get_insert_query(graph_iri: URIRef, data: Set) -> Tuple[List[str], int]:
    num_of_statements: int = len(data)
    if num_of_statements <= 0:
        return [], 0

    if num_of_statements <= MAX_TRIPLES_PER_QUERY:
        statements: str = _serialize_triples_to_nt(data)
        return [f"INSERT DATA {{ GRAPH <{graph_iri}> {{ {statements} }} }}"], num_of_statements

    chunks = _chunk_set(data, MAX_TRIPLES_PER_QUERY)
    queries = []
    for chunk in chunks:
        statements = _serialize_triples_to_nt(chunk)
        queries.append(f"INSERT DATA {{ GRAPH <{graph_iri}> {{ {statements} }} }}")
    return queries, num_of_statements


def _compute_graph_changes(entity: AbstractEntity, entity_type: str) -> Tuple[Set, Set, int, int]:
    """
    Computes the triples to insert and delete for an entity.

    Args:
        entity: The entity to analyze
        entity_type: Type of entity ("graph", "prov", or "metadata")

    Returns:
        Tuple of (triples_to_insert, triples_to_delete, added_count, removed_count)
    """
    if entity_type == "prov":
        triples = set(entity.g)
        return triples, set(), len(triples), 0

    # Deferred import to break circular dependency:
    # graph_entity → abstract_entity → support.support → (support/__init__) → query_utils → graph_entity
    from oc_ocdm.graph.graph_entity import GraphEntity  # noqa: E402

    assert isinstance(entity, GraphEntity)
    to_be_deleted: bool = entity.to_be_deleted
    preexisting_graph = entity.preexisting_graph

    if to_be_deleted:
        preexisting_triples = set(preexisting_graph)
        return set(), preexisting_triples, 0, len(preexisting_triples)

    preexisting_triples = set(preexisting_graph)
    current_triples = set(entity.g)

    if preexisting_triples == current_triples:
        return set(), set(), 0, 0

    removed_triples = preexisting_triples - current_triples
    added_triples = current_triples - preexisting_triples

    return added_triples, removed_triples, len(added_triples), len(removed_triples)


def get_update_query(entity: AbstractEntity, entity_type: str = "graph") -> Tuple[List[str], int, int]:
    to_insert, to_delete, n_added, n_removed = _compute_graph_changes(entity, entity_type)

    if n_added == 0 and n_removed == 0:
        return [], 0, 0

    graph_iri = entity.g.identifier
    assert isinstance(graph_iri, URIRef)

    delete_queries, _ = get_delete_query(graph_iri, to_delete)
    insert_queries, _ = get_insert_query(graph_iri, to_insert)

    return delete_queries + insert_queries, n_added, n_removed



# SPDX-FileCopyrightText: 2025 Arcangelo Massari <arcangelo.massari@unibo.it>
#
# SPDX-License-Identifier: ISC

"""
Plot generator for benchmark results.

Generates PNG visualizations from pytest-benchmark JSON output.
"""

import argparse
import json
from pathlib import Path
from typing import Any, Dict, List, Optional

import matplotlib.pyplot as plt
import matplotlib.ticker as ticker


class BenchmarkPlotGenerator:
    """
    Generates PNG visualizations from benchmark results.
    """

    COLORS = [
        "#2ecc71",
        "#3498db",
        "#e74c3c",
        "#9b59b6",
        "#f39c12",
        "#1abc9c",
        "#e67e22",
        "#34495e",
    ]

    def __init__(self, output_dir: str = "benchmarks/output/plots"):
        """
        Initialize the plot generator.

        Args:
            output_dir: Directory to save generated plots.
        """
        self.output_dir = Path(output_dir)
        self.output_dir.mkdir(parents=True, exist_ok=True)

    def load_benchmark_results(self, json_path: str) -> Dict[str, Any]:
        """
        Load benchmark results from pytest-benchmark JSON file.

        Args:
            json_path: Path to JSON file.

        Returns:
            Parsed benchmark data.
        """
        with open(json_path) as f:
            return json.load(f)

    def _extract_benchmarks_by_group(
        self, data: Dict[str, Any]
    ) -> Dict[str, List[Dict[str, Any]]]:
        """
        Group benchmarks by their group name.

        Args:
            data: Raw benchmark data.

        Returns:
            Dictionary mapping group names to benchmark lists.
        """
        groups = {}
        for benchmark in data["benchmarks"]:
            group = benchmark["group"]
            if group not in groups:
                groups[group] = []
            groups[group].append(benchmark)
        return groups

    def _get_benchmark_name(self, benchmark: Dict[str, Any]) -> str:
        """Extract a clean name from benchmark data."""
        name = benchmark["name"]
        if name.startswith("test_"):
            name = name[5:]
        return name

    def _get_param_value(self, benchmark: Dict[str, Any], param: str) -> Optional[Any]:
        """Extract parameter value from benchmark."""
        params = benchmark.get("params")
        if params is None:
            return None
        return params[param] if param in params else None

    def _get_group_parameters(self, benchmarks: List[Dict[str, Any]]) -> set:
        """Extract all parameter names used in a benchmark group."""
        params = set()
        for bm in benchmarks:
            bm_params = bm.get("params")
            if bm_params is not None:
                params.update(bm_params.keys())
        return params

    def plot_group_combined(
        self,
        data: Dict[str, Any],
        group: str,
        output_name: str,
    ):
        """
        Create a combined plot with comparison bar chart and scaling/throughput
        line charts for all parameters in a single PNG.

        Layout:
        - Row 1: Comparison bar chart (spans full width)
        - Row 2: For each parameter, scaling and throughput side by side

        Args:
            data: Benchmark data.
            group: Group name to plot.
            output_name: Output filename (without extension).
        """
        groups = self._extract_benchmarks_by_group(data)
        if group not in groups:
            print(f"Group '{group}' not found in benchmark data")
            return

        benchmarks = groups[group]
        group_params = list(self._get_group_parameters(benchmarks))

        # Determine layout: 1 row for comparison + 1 row for each parameter pair
        n_params = max(1, len(group_params))
        n_cols = n_params * 2  # scaling + throughput for each param

        # Create figure with GridSpec
        fig = plt.figure(figsize=(6 * n_params, 10))
        gs = fig.add_gridspec(2, n_cols, height_ratios=[1, 1])

        # Row 1: Comparison bar chart (spans all columns)
        ax_comparison = fig.add_subplot(gs[0, :])
        self._draw_comparison_subplot(ax_comparison, benchmarks, group)

        # Row 2: Scaling and throughput for each parameter
        if group_params:
            for i, param in enumerate(sorted(group_params)):
                ax_scaling = fig.add_subplot(gs[1, i * 2])
                ax_throughput = fig.add_subplot(gs[1, i * 2 + 1])
                self._draw_scaling_subplot(ax_scaling, benchmarks, param)
                self._draw_throughput_subplot(ax_throughput, benchmarks, param)

        plt.tight_layout()
        output_path = self.output_dir / f"{output_name}.png"
        plt.savefig(output_path, dpi=150)
        plt.close()
        print(f"Saved: {output_path}")

    def _draw_comparison_subplot(
        self, ax: plt.Axes, benchmarks: List[Dict[str, Any]], group: str
    ):
        """Draw comparison bar chart on given axes."""
        names = []
        means = []
        stddevs = []

        for bm in benchmarks:
            names.append(self._get_benchmark_name(bm))
            stats = bm["stats"]
            means.append(stats["mean"] * 1000)
            stddevs.append(stats["stddev"] * 1000)

        x = range(len(names))
        bars = ax.bar(x, means, yerr=stddevs, capsize=5, color=self.COLORS[0], alpha=0.8)

        ax.set_xlabel("Benchmark")
        ax.set_ylabel("Time (ms)")
        ax.set_title(f"{group} - Comparison")
        ax.set_xticks(x)
        ax.set_xticklabels(names, rotation=45, ha="right")

        for bar, mean in zip(bars, means):
            height = bar.get_height()
            ax.annotate(
                f"{mean:.2f}",
                xy=(bar.get_x() + bar.get_width() / 2, height),
                xytext=(0, 3),
                textcoords="offset points",
                ha="center",
                va="bottom",
                fontsize=8,
            )

    def _draw_scaling_subplot(
        self, ax: plt.Axes, benchmarks: List[Dict[str, Any]], param_name: str
    ):
        """Draw scaling line chart on given axes."""
        test_data = {}
        for bm in benchmarks:
            base_name = self._get_benchmark_name(bm)
            if "[" in base_name:
                base_name = base_name.split("[")[0]

            param_value = self._get_param_value(bm, param_name)
            if param_value is None:
                continue

            if base_name not in test_data:
                test_data[base_name] = {"x": [], "y": [], "err": []}

            stats = bm["stats"]
            test_data[base_name]["x"].append(param_value)
            test_data[base_name]["y"].append(stats["mean"] * 1000)
            test_data[base_name]["err"].append(stats["stddev"] * 1000)

        for i, (name, values) in enumerate(test_data.items()):
            sorted_data = sorted(zip(values["x"], values["y"], values["err"]))
            x = [d[0] for d in sorted_data]
            y = [d[1] for d in sorted_data]
            err = [d[2] for d in sorted_data]

            color = self.COLORS[i % len(self.COLORS)]
            ax.errorbar(x, y, yerr=err, marker="o", label=name, color=color, capsize=3)

        ax.set_xlabel(param_name.replace("_", " ").title())
        ax.set_ylabel("Time (ms)")
        ax.set_title(f"Scaling ({param_name})")
        ax.legend(loc="best", fontsize=7)
        ax.grid(True, alpha=0.3)

    def _draw_throughput_subplot(
        self, ax: plt.Axes, benchmarks: List[Dict[str, Any]], param_name: str
    ):
        """Draw throughput line chart on given axes."""
        test_data = {}
        for bm in benchmarks:
            base_name = self._get_benchmark_name(bm)
            if "[" in base_name:
                base_name = base_name.split("[")[0]

            param_value = self._get_param_value(bm, param_name)
            if param_value is None:
                continue

            if base_name not in test_data:
                test_data[base_name] = {"x": [], "y": []}

            stats = bm["stats"]
            mean_time = stats["mean"]
            throughput = param_value / mean_time

            test_data[base_name]["x"].append(param_value)
            test_data[base_name]["y"].append(throughput)

        for i, (name, values) in enumerate(test_data.items()):
            sorted_data = sorted(zip(values["x"], values["y"]))
            x = [d[0] for d in sorted_data]
            y = [d[1] for d in sorted_data]

            color = self.COLORS[i % len(self.COLORS)]
            ax.plot(x, y, marker="o", label=name, color=color)

        ax.set_xlabel(param_name.replace("_", " ").title())
        ax.set_ylabel("Throughput (entities/sec)")
        ax.set_title(f"Throughput ({param_name})")
        ax.legend(loc="best", fontsize=7)
        ax.grid(True, alpha=0.3)
        ax.yaxis.set_major_formatter(
            ticker.FuncFormatter(lambda x, _: format(int(x), ","))
        )

    def generate_all_plots(self, json_file: str):
        """
        Generate all standard plots from benchmark JSON file.

        Args:
            json_file: Path to benchmark JSON file.
        """
        json_path = Path(json_file)
        if not json_path.exists():
            print(f"File not found: {json_file}")
            return

        print(f"\nProcessing: {json_path.name}")
        data = self.load_benchmark_results(str(json_path))

        # Generate one combined plot per group
        groups = self._extract_benchmarks_by_group(data)

        for group_name in groups:
            safe_group_name = group_name.replace("/", "_").replace(" ", "_")
            self.plot_group_combined(data, group=group_name, output_name=safe_group_name)

    def generate_group_plot(self, json_file: str, group: str):
        """
        Generate plot for a specific benchmark group only.

        Args:
            json_file: Path to benchmark JSON file.
            group: Name of the benchmark group to plot.
        """
        json_path = Path(json_file)
        if not json_path.exists():
            print(f"File not found: {json_file}")
            return

        print(f"\nProcessing: {json_path.name} (group: {group})")
        data = self.load_benchmark_results(str(json_path))

        safe_group_name = group.replace("/", "_").replace(" ", "_")
        self.plot_group_combined(data, group=group, output_name=safe_group_name)


def main():
    """Generate plots from command line."""

    parser = argparse.ArgumentParser(description="Generate benchmark plots")
    parser.add_argument(
        "--input-file",
        default="benchmarks/output/json/benchmark.json",
        help="Path to benchmark JSON file",
    )
    parser.add_argument(
        "--output-dir",
        default="benchmarks/output/plots",
        help="Directory to save plots",
    )
    parser.add_argument(
        "--group",
        default=None,
        help="Generate plot for specific benchmark group only",
    )

    args = parser.parse_args()

    generator = BenchmarkPlotGenerator(output_dir=args.output_dir)
    if args.group:
        generator.generate_group_plot(args.input_file, args.group)
    else:
        generator.generate_all_plots(args.input_file)


if __name__ == "__main__":
    main()



#!/usr/bin/python

# SPDX-FileCopyrightText: 2025 Arcangelo Massari <arcangelo.massari@unibo.it>
#
# SPDX-License-Identifier: ISC

# -*- coding: utf-8 -*-
import unittest

from oc_ocdm.graph.graph_entity import GraphEntity
from oc_ocdm.graph.graph_set import GraphSet
from rdflib import XSD, Literal


class TestPointerList(unittest.TestCase):
    resp_agent = 'http://resp_agent.test/'

    @classmethod
    def setUpClass(cls) -> None:
        cls.graph_set = GraphSet("http://test/", "./info_dir/", "", False)

    def setUp(self):
        self.rp = self.graph_set.add_rp(self.resp_agent)
        self.pl = self.graph_set.add_pl(self.resp_agent)

    def test_has_content(self):
        content = "Content"
        result = self.pl.has_content(content)
        self.assertIsNone(result)

        triple = self.pl.res, GraphEntity.iri_has_content, Literal(content, datatype=XSD.string)
        self.assertIn(triple, self.pl.g)

    def test_contains_element(self):
        result = self.pl.contains_element(self.rp)
        self.assertIsNone(result)

        triple = self.pl.res, GraphEntity.iri_has_element, self.rp.res
        self.assertIn(triple, self.pl.g)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/python

# SPDX-FileCopyrightText: 2022-2026 Arcangelo Massari <arcangelo.massari@unibo.it>
#
# SPDX-License-Identifier: ISC

# -*- coding: utf-8 -*-
from __future__ import annotations

import os
import re
from dataclasses import dataclass
from datetime import datetime
from functools import lru_cache
from typing import TYPE_CHECKING
from rdflib import URIRef, Graph

if TYPE_CHECKING:
    from typing import Optional, List, Tuple, Match, Dict, Set
    from oc_ocdm.graph.entities.bibliographic.bibliographic_resource import BibliographicResource
    from oc_ocdm.graph.entities.bibliographic.responsible_agent import ResponsibleAgent
    from oc_ocdm.graph.entities.bibliographic.agent_role import AgentRole

from urllib.parse import quote


@dataclass
class ParsedURI:
    base_iri: str
    short_name: str
    prefix: str
    count: str
    is_prov: bool
    prov_subject_short_name: str
    prov_subject_prefix: str
    prov_subject_count: str

from rdflib import RDF, XSD, Literal
from typing import Union


def sparql_binding_to_term(binding: dict) -> Union[URIRef, Literal]:
    """Convert a SPARQL JSON result binding to an rdflib term.

    Per RDF 1.1, simple literals (no datatype, no language tag) are normalized to xsd:string.
    """
    if binding['type'] == 'uri':
        return URIRef(binding['value'])
    datatype = binding.get('datatype')
    lang = binding.get('xml:lang')
    if datatype is not None:
        datatype = URIRef(datatype)
    elif lang is None:
        datatype = XSD.string
    return Literal(binding['value'], datatype=datatype, lang=lang)


def normalize_graph_literals(g: Graph) -> None:
    triples_to_update = []
    for s, p, o in g:
        if isinstance(o, Literal) and o.datatype is None and o.language is None:
            triples_to_update.append((s, p, o, Literal(str(o), datatype=XSD.string)))
    for s, p, old_o, new_o in triples_to_update:
        g.remove((s, p, old_o))
        g.add((s, p, new_o))


def create_date(date_list: Optional[List[Optional[int]]] = None) -> Optional[str]:
    string: Optional[str] = None
    if date_list is not None:
        l_date_list: int = len(date_list)
        if l_date_list != 0 and date_list[0] is not None:
            if l_date_list == 3 and date_list[1] is not None and date_list[2] is not None and \
                    (date_list[1] != 1 or date_list[2] != 1):
                string = datetime(date_list[0], date_list[1], date_list[2]).strftime('%Y-%m-%d')
            elif l_date_list >= 2 and date_list[1] is not None:
                string = datetime(date_list[0], date_list[1], 1).strftime('%Y-%m')
            else:
                string = datetime(date_list[0], 1, 1).strftime('%Y')
    return string


def get_datatype_from_iso_8601(string: str) -> Tuple[URIRef, str]:
    # Keep only the "yyyy-mm-dd" part of the string
    string = string[:10]

    try:
        date_parts: List[int] = [int(s) for s in string.split(sep='-', maxsplit=2)]
    except ValueError:
        raise ValueError("The provided date string is not ISO-8601 compliant!")

    num_of_parts: int = len(date_parts)
    if num_of_parts == 3:
        return XSD.date, datetime(date_parts[0], date_parts[1], date_parts[2]).strftime('%Y-%m-%d')
    elif num_of_parts == 2:
        return XSD.gYearMonth, datetime(date_parts[0], date_parts[1], 1).strftime('%Y-%m')
    else:
        return XSD.gYear, datetime(date_parts[0], 1, 1).strftime('%Y')

def get_ordered_contributors_from_br(br: BibliographicResource,
                                     contributor_type: URIRef):

    ar_list: List[AgentRole] = br.get_contributors()

    list_id: int = 0
    heads: Dict[URIRef, Dict] = {}
    tails: Dict[URIRef, Dict] = {}
    sub_lists: List[Dict] = []
    from_id_to_res_in_heads: Dict[int, URIRef] = {}
    for ar in ar_list:
        role_type: Optional[URIRef] = ar.get_role_type()
        ra: Optional[ResponsibleAgent] = ar.get_is_held_by()
        next_ar: Optional[AgentRole] = ar.get_next()
        if next_ar is not None:
            next_ar_res: Optional[URIRef] = next_ar.res
        else:
            next_ar_res: Optional[URIRef] = None

        if role_type is not None and role_type == contributor_type and ra is not None:
            if next_ar_res is not None and next_ar_res in heads:
                sub_list: Dict = heads[next_ar_res]
                sub_list['list'].insert(0, ra)
                del heads[next_ar_res]
                heads[ar.res] = sub_list
                from_id_to_res_in_heads[sub_list['id']] = ar.res
            elif ar.res is not None and ar.res in tails:
                sub_list: Dict = tails[ar.res]
                sub_list['list'].append(ra)
                del tails[ar.res]

                if next_ar_res is not None:
                    tails[next_ar_res] = sub_list
            else:
                # This AR cannot be inserted into any list, so
                # we need to create an entirely new list for it:
                sub_list: Dict = {'id': list_id, 'list': [ra]}
                list_id += 1
                sub_lists.append(sub_list)

                heads[ar.res] = sub_list
                from_id_to_res_in_heads[sub_list['id']] = ar.res
                if next_ar_res is not None:
                    tails[next_ar_res] = sub_list

    ids_in_heads: Set[int] = {val['id'] for val in heads.values()}
    ids_in_tails: Set[int] = {val['id'] for val in tails.values()}
    diff_set: Set[int] = ids_in_heads - ids_in_tails
    if len(diff_set) == 0:
        # No contributor was found!
        return []
    elif len(diff_set) != 1:
        raise ValueError('A malformed list of AgentRole entities was given.')
    else:
        result_list: List[ResponsibleAgent] = []
        cur_id: int = diff_set.pop()
        already_merged_list_ids: Set[int] = set()
        finished: bool = False
        while not finished:
            found: bool = False
            if cur_id in from_id_to_res_in_heads:
                res: URIRef = from_id_to_res_in_heads[cur_id]
                subl: Dict = heads[res]
                subl_id: int = subl['id']
                if subl_id not in already_merged_list_ids:
                    found = True
                    already_merged_list_ids.add(subl_id)
                    result_list = subl['list'] + result_list

                    # Now we need to get the next cur_id value:
                    if res in tails:
                        cur_id = tails[res]['id']
                    else:
                        finished = True

            if not found:
                raise ValueError('A malformed list of AgentRole entities was given.')

        unmerged_list_ids: Set[int] = ids_in_heads - already_merged_list_ids
        if len(unmerged_list_ids) != 0:
            raise ValueError('A malformed list of AgentRole entities was given.')

        return result_list


def encode_url(u: str) -> str:
    return quote(u, "://")


def create_literal(g: Graph, res: URIRef, p: URIRef, s: str, dt: Optional[URIRef] = None, nor: bool = True) -> None:
    if not is_string_empty(s):
        dt = dt if dt is not None else XSD.string
        g.add((res, p, Literal(s, datatype=dt, normalize=nor)))


def create_type(g: Graph, res: URIRef, res_type: URIRef) -> None:
    g.add((res, RDF.type, res_type))


def is_string_empty(string: Optional[str]) -> bool:
    return string is None or string.strip() == ""


# Variable used in several functions
entity_regex: str = r"^(.+)/([a-z][a-z])/(0[1-9]+0)?((?:[1-9][0-9]*)|(?:\d+-\d+))$"
prov_regex: str = r"^(.+)/([a-z][a-z])/(0[1-9]+0)?((?:[1-9][0-9]*)|(?:\d+-\d+))/prov/([a-z][a-z])/([1-9][0-9]*)$"

_compiled_entity_regex = re.compile(entity_regex)
_compiled_prov_regex = re.compile(prov_regex)


@lru_cache(maxsize=4096)
def parse_uri(res: URIRef) -> ParsedURI:
    string_iri = str(res)
    if "/prov/" in string_iri:
        match = _compiled_prov_regex.match(string_iri)
        if match:
            return ParsedURI(
                base_iri=match.group(1),
                short_name=match.group(5),
                prefix="",
                count=match.group(6),
                is_prov=True,
                prov_subject_short_name=match.group(2),
                prov_subject_prefix=match.group(3) or "",
                prov_subject_count=match.group(4),
            )
    else:
        match = _compiled_entity_regex.match(string_iri)
        if match:
            return ParsedURI(
                base_iri=match.group(1),
                short_name=match.group(2),
                prefix=match.group(3) or "",
                count=match.group(4),
                is_prov=False,
                prov_subject_short_name="",
                prov_subject_prefix="",
                prov_subject_count="",
            )
    return ParsedURI("", "", "", "", False, "", "", "")


def get_base_iri(res: URIRef) -> str:
    return parse_uri(res).base_iri


def get_short_name(res: URIRef) -> str:
    return parse_uri(res).short_name


def get_prefix(res: URIRef) -> str:
    return parse_uri(res).prefix


def get_count(res: URIRef) -> str:
    return parse_uri(res).count


def get_resource_number(res: URIRef) -> int:
    parsed = parse_uri(res)
    count = parsed.prov_subject_count if parsed.is_prov else parsed.count
    return int(count) if count else 0


def find_local_line_id(res: URIRef, n_file_item: int = 1) -> int:
    cur_number: int = get_resource_number(res)

    cur_file_split: int = 0
    while True:
        if cur_number > cur_file_split:
            cur_file_split += n_file_item
        else:
            cur_file_split -= n_file_item
            break

    return cur_number - cur_file_split


def find_paths(res: URIRef, base_dir: str, base_iri: str, default_dir: str, dir_split: int,
               n_file_item: int, is_json: bool = True, process_id: int|str|None = None) -> Tuple[str, str]:
    """
    This function is responsible for looking for the correct JSON file that contains the data related to the
    resource identified by the variable 'string_iri'. This search takes into account the organisation in
    directories and files, as well as the particular supplier prefix for bibliographic entities, if specified.
    In case no supplier prefix is specified, the 'default_dir' (usually set to "_") is used instead.
    """
    string_iri: str = str(res)
    process_id_str: str = f"_{process_id}" if process_id else ""

    if is_dataset(res):
        cur_dir_path: str = (base_dir + re.sub(r"^%s(.*)$" % base_iri, r"\1", string_iri))[:-1]
        cur_file_path: str = cur_dir_path + os.sep + "index" + process_id_str + ".json"
        return cur_dir_path, cur_file_path

    parsed = parse_uri(res)
    cur_number: int = int(parsed.prov_subject_count) if parsed.is_prov else int(parsed.count)

    cur_file_split: int = 0
    while cur_number > cur_file_split:
        cur_file_split += n_file_item

    if dir_split and not string_iri.startswith(base_iri + "prov/"):
        cur_split: int = 0
        while cur_number > cur_split:
            cur_split += dir_split

        if parsed.is_prov:
            sub_folder = parsed.prov_subject_prefix or default_dir or "_"
            file_extension = '.json' if is_json else '.nq'
            cur_dir_path = base_dir + parsed.prov_subject_short_name + os.sep + sub_folder + \
                os.sep + str(cur_split) + os.sep + str(cur_file_split) + os.sep + "prov"
            cur_file_path = cur_dir_path + os.sep + parsed.short_name + process_id_str + file_extension
        else:
            sub_folder = parsed.prefix or default_dir or "_"
            file_extension = '.json' if is_json else '.nt'
            cur_dir_path = base_dir + parsed.short_name + os.sep + sub_folder + os.sep + str(cur_split)
            cur_file_path = cur_dir_path + os.sep + str(cur_file_split) + process_id_str + file_extension
    elif dir_split == 0:
        if parsed.is_prov:
            sub_folder = parsed.prov_subject_prefix or default_dir or "_"
            file_extension = '.json' if is_json else '.nq'
            cur_dir_path = base_dir + parsed.prov_subject_short_name + os.sep + sub_folder + \
                os.sep + str(cur_file_split) + os.sep + "prov"
            cur_file_path = cur_dir_path + os.sep + parsed.short_name + process_id_str + file_extension
        else:
            sub_folder = parsed.prefix or default_dir or "_"
            file_extension = '.json' if is_json else '.nt'
            cur_dir_path = base_dir + parsed.short_name + os.sep + sub_folder
            cur_file_path = cur_dir_path + os.sep + str(cur_file_split) + process_id_str + file_extension
    else:
        file_extension = '.json' if is_json else '.nq'
        cur_dir_path = base_dir + parsed.short_name
        cur_file_path = cur_dir_path + os.sep + parsed.prefix + parsed.count + process_id_str + file_extension

    return cur_dir_path, cur_file_path

def has_supplier_prefix(res: URIRef, base_iri: str) -> bool:
    string_iri: str = str(res)
    return re.search(r"^%s[a-z][a-z]/0" % base_iri, string_iri) is not None

def build_graph_from_results(results: List[Dict]) -> Graph:
    graph = Graph()
    for triple in results:
        s = sparql_binding_to_term(triple['s'])
        p = sparql_binding_to_term(triple['p'])
        o = sparql_binding_to_term(triple['o'])
        graph.add((s, p, o))
    return graph


def is_dataset(res: URIRef) -> bool:
    string_iri: str = str(res)
    return re.search(r"^.+/[0-9]+(-[0-9]+)?(/[0-9]+)?$", string_iri) is None


#!/usr/bin/python

# SPDX-FileCopyrightText: 2025 Arcangelo Massari <arcangelo.massari@unibo.it>
#
# SPDX-License-Identifier: ISC

# -*- coding: utf-8 -*-
import unittest

from oc_ocdm.graph.graph_entity import GraphEntity
from oc_ocdm.graph.graph_set import GraphSet
from rdflib import RDF, XSD, Literal, URIRef


class TestResourceEmbodiment(unittest.TestCase):
    resp_agent = 'http://resp_agent.test/'

    @classmethod
    def setUpClass(cls) -> None:
        cls.graph_set = GraphSet("http://test/", "./info_dir/", "", False)

    def setUp(self):
        self.re = self.graph_set.add_re(self.resp_agent)

    def test_has_media_type(self):
        media_type = URIRef("http://test/MediaType")
        result = self.re.has_media_type(media_type)
        self.assertIsNone(result)

        triple = self.re.res, GraphEntity.iri_has_format, media_type
        self.assertIn(triple, self.re.g)

    def test_has_starting_page(self):
        starting_page = "15"
        result = self.re.has_starting_page(starting_page)
        self.assertIsNone(result)

        triple = self.re.res, GraphEntity.iri_starting_page, Literal(starting_page, datatype=XSD.string)
        self.assertIn(triple, self.re.g)

    def test_has_ending_page(self):
        ending_page = "288"
        result = self.re.has_ending_page(ending_page)
        self.assertIsNone(result)

        triple = self.re.res, GraphEntity.iri_ending_page, Literal(ending_page, datatype=XSD.string)
        self.assertIn(triple, self.re.g)

    def test_has_url(self):
        url = URIRef("http://test/url")
        result = self.re.has_url(url)
        self.assertIsNone(result)

        triple = self.re.res, GraphEntity.iri_has_url, url
        self.assertIn(triple, self.re.g)

    def test_create_digital_embodiment(self):
        result = self.re.create_digital_embodiment()
        self.assertIsNone(result)

        triple = self.re.res, RDF.type, GraphEntity.iri_digital_manifestation
        self.assertIn(triple, self.re.g)

    def test_create_print_embodiment(self):
        result = self.re.create_print_embodiment()
        self.assertIsNone(result)

        triple = self.re.res, RDF.type, GraphEntity.iri_print_object
        self.assertIn(triple, self.re.g)


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/python

# SPDX-FileCopyrightText: 2025 Arcangelo Massari <arcangelo.massari@unibo.it>
#
# SPDX-License-Identifier: ISC

# -*- coding: utf-8 -*-
import unittest
from rdflib import Graph, URIRef, Literal
from rdflib.namespace import RDF, DCTERMS

from oc_ocdm.graph.graph_set import GraphSet
from oc_ocdm.prov.prov_set import ProvSet
from oc_ocdm.support.query_utils import (
    get_update_query, get_insert_query, get_delete_query, _compute_graph_changes
)


class TestQueryUtils(unittest.TestCase):
    """Test suite for query_utils functions."""

    def setUp(self):
        self.base_iri = "https://test.org/"
        self.graph_set = GraphSet(self.base_iri, "", "060", False)

    def test_get_insert_query_empty_set(self):
        """Test get_insert_query with empty set returns empty list."""
        graph_iri = URIRef("https://test.org/graph/1")
        empty_set = set()

        queries, count = get_insert_query(graph_iri, empty_set)

        self.assertEqual(queries, [])
        self.assertEqual(count, 0)

    def test_get_insert_query_with_triples(self):
        """Test get_insert_query with triples generates correct query."""
        graph_iri = URIRef("https://test.org/graph/1")
        subject = URIRef("https://test.org/resource/1")
        triples = {
            (subject, RDF.type, URIRef("https://test.org/Class")),
            (subject, DCTERMS.title, Literal("Test")),
        }

        queries, count = get_insert_query(graph_iri, triples)

        self.assertEqual(len(queries), 1)
        self.assertIn("INSERT DATA", queries[0])
        self.assertIn(str(graph_iri), queries[0])
        self.assertEqual(count, 2)

    def test_get_delete_query_empty_set(self):
        """Test get_delete_query with empty set returns empty list."""
        graph_iri = URIRef("https://test.org/graph/1")
        empty_set = set()

        queries, count = get_delete_query(graph_iri, empty_set)

        self.assertEqual(queries, [])
        self.assertEqual(count, 0)

    def test_get_delete_query_with_triples(self):
        """Test get_delete_query with triples generates correct query."""
        graph_iri = URIRef("https://test.org/graph/1")
        subject = URIRef("https://test.org/resource/1")
        triples = {(subject, RDF.type, URIRef("https://test.org/Class"))}

        queries, count = get_delete_query(graph_iri, triples)

        self.assertEqual(len(queries), 1)
        self.assertIn("DELETE DATA", queries[0])
        self.assertIn(str(graph_iri), queries[0])
        self.assertEqual(count, 1)

    def test_get_update_query_unchanged_entity(self):
        """Test get_update_query returns empty for unchanged entity."""
        br = self.graph_set.add_br(self.base_iri + "br/1")
        br.has_title("Test Title")
        br.preexisting_graph = Graph(identifier=br.g.identifier)
        for triple in br.g:
            br.preexisting_graph.add(triple)

        queries, added, removed = get_update_query(br, entity_type="graph")

        self.assertEqual(queries, [])
        self.assertEqual(added, 0)
        self.assertEqual(removed, 0)

    def test_get_update_query_new_entity(self):
        """Test get_update_query for new entity generates INSERT."""
        br = self.graph_set.add_br(self.base_iri + "br/1")
        br.has_title("New Title")

        queries, added, removed = get_update_query(br, entity_type="graph")

        self.assertTrue(len(queries) >= 1)
        self.assertTrue(any("INSERT DATA" in q for q in queries))
        self.assertEqual(added, 2)
        self.assertEqual(removed, 0)

    def test_get_update_query_modified_entity(self):
        """Test get_update_query for modified entity with changes."""
        br = self.graph_set.add_br(self.base_iri + "br/1")
        br.has_title("Original Title")

        br.preexisting_graph = Graph(identifier=br.g.identifier)
        for triple in br.g:
            br.preexisting_graph.add(triple)

        br.g.add((br.res, URIRef("http://example.org/newProp"), Literal("New Value")))

        queries, added, removed = get_update_query(br, entity_type="graph")

        self.assertTrue(len(queries) >= 1)
        self.assertTrue(any("INSERT DATA" in q for q in queries))
        self.assertEqual(added, 1)
        self.assertEqual(removed, 0)

    def test_get_update_query_deleted_entity(self):
        """Test get_update_query for deleted entity generates DELETE."""
        br = self.graph_set.add_br(self.base_iri + "br/1")
        br.has_title("To Delete")
        br.preexisting_graph = Graph(identifier=br.g.identifier)
        for triple in br.g:
            br.preexisting_graph.add(triple)

        br.mark_as_to_be_deleted()

        queries, added, removed = get_update_query(br, entity_type="graph")

        self.assertTrue(len(queries) >= 1)
        self.assertTrue(any("DELETE DATA" in q for q in queries))
        self.assertFalse(any("INSERT DATA" in q for q in queries))
        self.assertEqual(added, 0)
        self.assertEqual(removed, 2)

    def test_get_update_query_prov_entity_optimization(self):
        """Test get_update_query for prov entity skips graph_diff."""
        br = self.graph_set.add_br(self.base_iri + "br/1")
        br.has_title("Test")

        prov_set = ProvSet(self.graph_set, self.base_iri)
        se = prov_set.add_se(br)
        se.has_description("Creation")

        queries, added, removed = get_update_query(se, entity_type="prov")

        self.assertTrue(len(queries) >= 1)
        self.assertTrue(any("INSERT DATA" in q for q in queries))
        self.assertFalse(any("DELETE DATA" in q for q in queries))
        self.assertEqual(added, len(se.g))
        self.assertEqual(removed, 0)

    def test_get_update_query_early_exit_same_length(self):
        """Test early-exit optimization when graphs have same length."""
        br = self.graph_set.add_br(self.base_iri + "br/1")
        br.has_title("Test Title")
        br.preexisting_graph = Graph(identifier=br.g.identifier)
        for triple in br.g:
            br.preexisting_graph.add(triple)

        queries, added, removed = get_update_query(br, entity_type="graph")

        self.assertEqual(queries, [])
        self.assertEqual(added, 0)
        self.assertEqual(removed, 0)

    def test_get_update_query_different_length_graphs(self):
        """Test that graphs with different lengths generate queries."""
        br = self.graph_set.add_br(self.base_iri + "br/1")
        br.has_title("Original")
        br.preexisting_graph = Graph(identifier=br.g.identifier)
        for triple in br.g:
            br.preexisting_graph.add(triple)

        br.has_subtitle("New Subtitle")

        queries, added, removed = get_update_query(br, entity_type="graph")

        self.assertTrue(len(queries) >= 1)
        self.assertTrue(any("INSERT DATA" in q for q in queries))
        self.assertEqual(added, 1)
        self.assertEqual(removed, 0)

    def test_get_update_query_entity_with_only_additions(self):
        """Test entity with only new triples generates INSERT only."""
        br = self.graph_set.add_br(self.base_iri + "br/1")
        br.has_title("Original")
        br.preexisting_graph = Graph(identifier=br.g.identifier)
        for triple in br.g:
            br.preexisting_graph.add(triple)

        br.has_subtitle("Additional Info")

        queries, added, removed = get_update_query(br, entity_type="graph")

        self.assertTrue(len(queries) >= 1)
        self.assertTrue(any("INSERT DATA" in q for q in queries))
        self.assertFalse(any("DELETE DATA" in q for q in queries))
        self.assertEqual(added, 1)
        self.assertEqual(removed, 0)

    def test_compute_graph_changes_prov_entity(self):
        """Test _compute_graph_changes with provenance entity."""
        br = self.graph_set.add_br(self.base_iri + "br/1")
        br.has_title("Test")

        prov_set = ProvSet(self.graph_set, self.base_iri)
        se = prov_set.add_se(br)
        se.has_description("Creation")

        to_insert, to_delete, added, removed = _compute_graph_changes(se, "prov")

        self.assertEqual(len(to_insert), len(se.g))
        self.assertEqual(len(to_delete), 0)
        self.assertEqual(added, len(se.g))
        self.assertEqual(removed, 0)

    def test_compute_graph_changes_deleted_entity(self):
        """Test _compute_graph_changes with deleted entity."""
        br = self.graph_set.add_br(self.base_iri + "br/1")
        br.has_title("To Delete")
        br.preexisting_graph = Graph(identifier=br.g.identifier)
        for triple in br.g:
            br.preexisting_graph.add(triple)

        original_count = len(br.preexisting_graph)
        br.mark_as_to_be_deleted()

        to_insert, to_delete, added, removed = _compute_graph_changes(br, "graph")

        self.assertEqual(len(to_insert), 0)
        self.assertEqual(len(to_delete), original_count)
        self.assertEqual(added, 0)
        self.assertEqual(removed, original_count)

    def test_compute_graph_changes_unchanged_entity(self):
        """Test _compute_graph_changes with unchanged entity."""
        br = self.graph_set.add_br(self.base_iri + "br/1")
        br.has_title("Test")
        br.preexisting_graph = Graph(identifier=br.g.identifier)
        for triple in br.g:
            br.preexisting_graph.add(triple)

        to_insert, to_delete, added, removed = _compute_graph_changes(br, "graph")

        self.assertEqual(len(to_insert), 0)
        self.assertEqual(len(to_delete), 0)
        self.assertEqual(added, 0)
        self.assertEqual(removed, 0)

    def test_compute_graph_changes_modified_entity(self):
        """Test _compute_graph_changes with modified entity."""
        br = self.graph_set.add_br(self.base_iri + "br/1")
        br.has_title("Original")
        br.preexisting_graph = Graph(identifier=br.g.identifier)
        for triple in br.g:
            br.preexisting_graph.add(triple)

        br.has_subtitle("New Subtitle")

        to_insert, to_delete, added, removed = _compute_graph_changes(br, "graph")

        self.assertEqual(len(to_insert), 1)
        self.assertEqual(len(to_delete), 0)
        self.assertEqual(added, 1)
        self.assertEqual(removed, 0)


if __name__ == '__main__':
    unittest.main()



# SPDX-FileCopyrightText: 2025 Arcangelo Massari <arcangelo.massari@unibo.it>
#
# SPDX-License-Identifier: ISC

import pytest
from rdflib import Graph

from oc_ocdm.support.query_utils import _compute_graph_changes

from benchmarks.conftest import BENCHMARK_ROUNDS, create_populated_graph_set


class MockEntity:
    def __init__(self, current_graph, preexisting_graph):
        self.g = current_graph
        self.preexisting_graph = preexisting_graph
        self.to_be_deleted = False


class TestGraphDiff:

    @pytest.mark.benchmark(group="graph_diff")
    @pytest.mark.parametrize("entity_count", [50, 100, 150])
    def test_compute_graph_changes_modified(
        self, benchmark, redis_counter_handler, entity_count
    ):
        def setup():
            graph_set, brs = create_populated_graph_set(
                redis_counter_handler, entity_count
            )
            entities = []
            for entity in graph_set.res_to_entity.values():
                preexisting = Graph()
                for triple in entity.g:
                    preexisting.add(triple)
                mock = MockEntity(entity.g, preexisting)
                entities.append(mock)
            for i, br in enumerate(brs):
                br.has_title(f"Modified Title {i}")
            return (entities,), {}

        def compute_changes(entities):
            results = []
            for entity in entities:
                result = _compute_graph_changes(entity, "graph")
                results.append(result)
            return results

        result = benchmark.pedantic(
            compute_changes, setup=setup, rounds=BENCHMARK_ROUNDS
        )
        assert len(result) >= entity_count * 14
        entities_with_changes = sum(
            1 for to_insert, to_delete, n_added, n_removed in result
            if n_added > 0 or n_removed > 0
        )
        assert entities_with_changes == entity_count, (
            f"Expected {entity_count} entities with changes, "
            f"got {entities_with_changes}"
        )



#!/usr/bin/python

# SPDX-FileCopyrightText: 2025 Arcangelo Massari <arcangelo.massari@unibo.it>
#
# SPDX-License-Identifier: ISC

# -*- coding: utf-8 -*-
import unittest


class TestGraphEntity(unittest.TestCase):
    pass


if __name__ == '__main__':
    unittest.main()
